#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "NimBLEDevice.h"
#include "GPIO_drv.hpp"
#include "SPI_drv.hpp"

extern "C" {void app_main(void);}

static NimBLEServer* pServer;
GPIO_drv latch;
SPI_drv spi;
std::uint8_t chislo = 0xFF;

// FreeRTOS Task
void writeToShift(void *pvParameters)  
{
  (void) pvParameters;

    latch.setLvl(Lvl::reset);
    spi.transfer(&chislo, 1);
    latch.setLvl(Lvl::set);
}

// BLE Callbacks
class ServerCallbacks: public NimBLEServerCallbacks {
    void onConnect(NimBLEServer* pServer) {
        printf("Client connected\n");
        
        TaskHandle_t xHandle = NULL;
        xTaskCreate( writeToShift, "SPISHIFT", 128, NULL, 1, &xHandle );
            if( xHandle != NULL )
        vTaskDelete(xHandle);
        
        NimBLEDevice::startAdvertising();
    };

    void onDisconnect(NimBLEServer* pServer) {
        printf("Client disconnected - start advertising\n");

        TaskHandle_t xHandle = NULL;
        xTaskCreate( writeToShift, "SPISHIFT", 128, NULL, 1, &xHandle );
            if( xHandle != NULL )
        vTaskDelete(xHandle);
        
        NimBLEDevice::startAdvertising();
    };
};

void app_main(void)
{
    // Init Ports
    latch.init(GPIO_NUM_2, GPIO_MODE_OUTPUT);
    spi.init(SPI_HOST, 1000000, GPIO_NUM_5);
     
    //Init Bluetooth
    NimBLEDevice::init("NimBLE");

    pServer = NimBLEDevice::createServer();
    pServer->setCallbacks(new ServerCallbacks());

    NimBLEAdvertising* pAdvertising = NimBLEDevice::getAdvertising();
    pAdvertising->start();
}
